var _data = loadJsonFile("data");
var _selectView;
var selectedLang = 'en';
var isDrawerOpened = false;
var _focussedTextField;

function init() {
	$.window.open();
	retriveData();
	populateListView();
}
init();

/**
 * load JSON file
 */
function loadJsonFile(name) {
	var file = Ti.Filesystem.getFile(Ti.Filesystem.resourcesDirectory, name + '.json');
	if (file.exists()) {
  		var dataSrc = Ti.Platform.osname === 'android' ? ''+file.read() : file.read();
  		data = JSON.parse(dataSrc);
  		return data;
	}
}

/**
 * populate list view with content from JSON file
 *
 * - one section per question
 * - sub-questions are added to same section
 */
function populateListView() {
    var sections = [];
    for (var i=0; i<_data.length; i++) {
        var section = createSection(_data[i]);
        sections.push(section);
    }
	$.listView.sections = sections;
}

/**
 * create list view section
 * i.e. populate one question (and any sub-questions)
 */
function createSection(question) {
    var items = [];
    var section = Ti.UI.createListSection();
    var item = createItem(question, "template_");
    items.push(item);
    if (question.subQuestions && question.value === question.subQuestions.on) {
        for (var j=0; j<question.subQuestions.questions.length; j++) {
            var item = createItem(question.subQuestions.questions[j], "template_sub_");
            items.push(item);
        }
    }
    else if(question.subQuestions && question.subQuestions.questions.length > 0){
    	//Remove the answers if the user hides child question
    	for(var i=0;i<question.subQuestions.questions.length;i++){
    		question.subQuestions.questions[i].value = "";
    	}
    	
    }
    section.setItems(items);
    return section;
}

/**
 * create list view item
 * returns list view items (i.e. TableView row) for question or sub-question
 */
function createItem(question, templatePrefix) {
    var item = {
        template: templatePrefix + question.type,
        questionIdLabel: { text: question.uiQuestionID },
        textLabel: { text: question.questionText[selectedLang]},
        mandatoryLabel: { text: (question.mandatory ? "*" : "" )}
    };

    // template-specific features
    if (question.type == "yes_no") {
        if (question.value === true) {
            item['yesButton'] = { borderColor: "transparent", backgroundColor: "transparent", backgroundImage : "/images/eApp/icons/yn_selected.png", color : Alloy.Globals.COLORS.ACE_WHITE }
        } else if (question.value == false) {
            item['noButton'] = { borderColor: "transparent", backgroundColor: "transparent", backgroundImage : "/images/eApp/icons/yn_selected.png", color : Alloy.Globals.COLORS.ACE_WHITE  }
        }
    } else if (question.type == "textfield") {
    	var answerTextFieldProperties = { value: question.value, borderColor: "transparent", backgroundColor: "transparent" };
        if (question.valid === true) {
            answerTextFieldProperties.borderColor = "transparent";
            answerTextFieldProperties.backgroundColor = "transparent";
        } else if (question.valid == false) {
            answerTextFieldProperties.borderColor = "transparent";
            answerTextFieldProperties.backgroundColor = "transparent";
        }
        item['answerTextField'] = answerTextFieldProperties;
      
    } else if (question.type == "select") {
        if (question.value) {
            item['selectLabel'] = { text: question.title };
        }
    }

    return item;
}

/**
 * replace list view section
 * replaces question to update content or show/hide sub-questions
 */
function replaceSection(index) {
	if (_focussedTextField) {
		_focussedTextField.blur();
	}
    var section = createSection(_data[index],index);
    $.listView.replaceSectionAt(index, section);
}

/**
 * yes button click handler
 * updates JSON and reloads list view section containing question
 */
function yesButtonClick(e) {
    e.cancelBubble = true;
    e.bubblesParent = false;
    if (e.itemIndex === 0) { // question selected
        _data[e.sectionIndex].value = true;
    } else { // sub-question selected
        _data[e.sectionIndex].subQuestions.questions[e.itemIndex-1].value = true;
    }
    replaceSection(e.sectionIndex);
}

/**
 * no button click handler
 * updates JSON and reloads list view section containing question
 */
function noButtonClick(e) {
    // Ti.API.debug(JSON.stringify(e, null, 4));
    e.cancelBubble = true;
    e.bubblesParent = false;
    if (e.itemIndex === 0) { // question selected
        _data[e.sectionIndex].value = false;
    } else { // sub-question selected
        _data[e.sectionIndex].subQuestions.questions[e.itemIndex-1].value = false;
    }
    replaceSection(e.sectionIndex);
}

/**
 * text field change handler
 * - called on change to text in textfield question type
 * - calls validator function and updates style
 */
function textFieldChange(e) {
    // Ti.API.debug(JSON.stringify(e, null, 4));
    e.cancelBubble = true;
    e.bubblesParent = false;
    var question;
    if (e.itemIndex === 0) { // question selected
        question = _data[e.sectionIndex];
    } else { // sub-question selected
        question = _data[e.sectionIndex].subQuestions.questions[e.itemIndex-1];
    }
    question.value = e.value;

    if (question.validator) { // validator function
        if (validators[question.validator](e.value)) { // <- calls validator function === isTitanium(text)
            question.valid = true;
            e.source.borderColor = "#00dd00";
            e.source.backgroundColor = "#f0fff0";
        } else {
            question.valid = false;
            e.source.borderColor = "#dd0000";
            e.source.backgroundColor = "#fff0f0";
        }
    }
}

function textFieldBlur(e) {
	var blueLineNormalStyle = $.createStyle({
		classes : "blueLineViewNormalStyle",
		apiName : "Label"
	}); 

	//$.blueLineView.applyProperties(blueLineNormalStyle);
    Ti.API.debug(JSON.stringify(e, null, 4));
    replaceSection(e.sectionIndex);
    
}

function textFieldReturn(e) {
    // Ti.API.debug(JSON.stringify(e, null, 4));
}

function textFieldFocus(e){
	Ti.API.info('Focus:: '+JSON.stringify(e));
	_focussedTextField = e.source;
	var blueLineSelectedStyle = $.createStyle({
		classes : "blueLineViewSelectedStyle",
		apiName : "Label"
	}); 
	var q=null;
	if (e.itemIndex === 0) { // question selected
        q = _data[e.sectionIndex];
    } else { // sub-question selected
        q = _data[e.sectionIndex].subQuestions.questions[e.itemIndex-1];
    }
 	var data = {"maxChar":q.maxLen.value,"detailsValidation":[],"minChar":q.minLen.value};
	//showValidationDrawer(data);
	//e.source.borderColor = "#00dd00";
		
	//$.blueLineView.applyProperties(blueLineSelectedStyle);
}
/**
 * select label click handler
 * opens and populates select overlay
 */
function selectLabelClick(e) {
    // Ti.API.debug(JSON.stringify(e, null, 4));
    if(!isDrawerOpened){
    	isDrawerOpened = true;
	    e.cancelBubble = true;
	    e.bubblesParent = false;
	
	    var question;
	    if (e.itemIndex === 0) { // question selected
	        question = _data[e.sectionIndex];
	    } else { // sub-question selected
	        question = _data[e.sectionIndex].subQuestions.questions[e.itemIndex-1];
	    }
	
	    _selectView = Alloy.createController('view/select', _.extend({sectionIndex: e.sectionIndex, itemIndex: e.itemIndex, question: question, callback: selectCallback}));
	    _selectView.getView().right = -471;
	    //get the current window to attach the dropdwon controller
	    /*var navigationControl = require("common/NavigationControl");
		currentLoaderWindow = navigationControl.getCurrentController();
		navigationControl = null;
	    currentLoaderWindow.add(_selectView.getView());*/
	    $.window.add(_selectView.getView());
	    var animation = Ti.UI.createAnimation({
	        duration: 300,
	        right: 0
	    });
	    _selectView.getView().animate(animation);
    }
}

/**
 * select callback
 * - called by select controller when option is selected
 * - updates JSON and replaces section
 */

function selectCallback(sectionIndex, itemIndex, question) {
    if (itemIndex === 0) { // question selected
        _data[sectionIndex] = question;
    } else { // sub-question selected
        _data[sectionIndex].subQuestions.questions[itemIndex-1] = question;
    }
    // Ti.API.debug(JSON.stringify(_data[index], null, 4));
    replaceSection(sectionIndex);

    var animation = Ti.UI.createAnimation({
        duration: 300,
        right: -471
    });
    _selectView.getView().animate(animation);
    isDrawerOpened = false;
}

/**
 * validator functions
 */
var validators = {
    isTitanium : function(text) {
        return text.trim().toLowerCase() === "titanium";
    },
    isiOS : function(text) {
        return text.trim().toLowerCase() === "ios";
    }
}

function saveData(e){
	console.log("testing......... handlePrevTouchStart"+JSON.stringify(_data));
	var answers = [];
	for(var i=0;i<_data.length;i++){
		var ans = {};
		ans.key = _data[i].id;
		ans.parentkey = _data[i].pid;
		ans.value = _data[i].value;
		answers.push(ans);
		if(_data[i].subQuestions && _data[i].subQuestions.questions && _data[i].subQuestions.questions.length > 0){
			for(var j=0;j<_data[i].subQuestions.questions.length;j++){
				console.log("_data[i].subQuestions.questions="+JSON.stringify(_data[i].subQuestions.questions));
				var ans = {};
				ans.key = _data[i].subQuestions.questions[j].id;
				ans.parentkey = _data[i].subQuestions.questions[j].pid;
				ans.value = _data[i].subQuestions.questions[j].value;
				answers.push(ans);
			}	
		}
	}
	console.log("testing......... answers"+JSON.stringify(answers));
	Ti.App.Properties.setString('dataSaved', JSON.stringify(answers));
	alert('data saved');
}

function retriveData(){
	var answers = Ti.App.Properties.getString('dataSaved', null);
	answers = JSON.parse(answers);
	if(answers){
		for(var i=0;i<_data.length;i++){
  			//get the value from  the collection based to id and pid and then set it to json data
  			//Parent questions
  			var ans = _.findWhere(answers, {key: _data[i].id});
  			if(ans){
  				_data[i].value = ans.value;
  				if(_data[i].type == "select"){
  					_data[i].title = getPickerDisplayValue(_data[i].fieldValues,ans.value);
  				}
  			}
  			//Child Questions
  			if(_data[i].subQuestions && _data[i].subQuestions.questions &&  _data[i].subQuestions.questions.length > 0){
  				for(var k=0;k<_data[i].subQuestions.questions.length;k++){
	  				var subQans = _.findWhere(answers, {key: _data[i].subQuestions.questions[k].id,parentkey : _data[i].id});
		  			if(subQans){
		  				_data[i].subQuestions.questions[k].value = subQans.value;
		  				if(_data[i].subQuestions.questions[k].type == "select"){
		  					_data[i].subQuestions.questions[k].title = getPickerDisplayValue(_data[i].subQuestions.questions[k].fieldValues,subQans.value);
		  				}
		  				
		  			}
	  			}
  			}
  			
  		}
	}
	console.log("_data="+JSON.stringify(_data));
}

function openErrorList(){
	
}

function openCloseActionBar(){
	
}

function completeEapp(){
	
}

function handleBtnTouch(){
	
}

function handlePrevTouchEnd(){
	
}

function handleNextTouchEnd(){
	
}



function handlePrevTouchStart(){
	
}

function handleNextTouchStart(){
	
}

function doPostLayout(){
	
}

function getPickerDisplayValue(fieldValues,selectedVal){
	var ddOptions = require("util/QnIUtils").getDropdownOptions(fieldValues);
	var selectedOption = _.findWhere(ddOptions, {"value": selectedVal+""});
	if(selectedOption && selectedOption.displayValueLang1){
		return selectedOption.displayValueLang1;
	}
	else{
		return "";
	}
	 
}

function goBack(){
	$.window.close();
}

function showValidationDrawer(validationObj){
	$.validationBar.visible = true;
	var topValidationDrawer = Alloy.createController("view/TopValidationBar");
	$.validationBar.add(topValidationDrawer.getView());
	//console.log("showValidationDrawer validationObj="+JSON.stringify(validationObj));
	topValidationDrawer.animateDown(validationObj);
	
}
